@@ -104,11 +104,32 @@ gok_discard_focus_filter (GdkXEvent *xevent,
 			  gpointer  data)
 {
   XEvent *xev = (XEvent *)xevent;	
+  GList *list, *l;
+  gint n_mapped;
+  GtkWindow *window;
+  gboolean has_focus;
 
   if (xev->xany.type == ClientMessage &&
       (Atom) xev->xclient.data.l[0] == gdk_x11_atom_to_xatom (
 	      gdk_atom_intern ("WM_TAKE_FOCUS", False)))
     {
+      list = gtk_window_list_toplevels ();
+      n_mapped = 0;
+      has_focus = FALSE;
+      for (l = list; l; l = l->next)
+        {
+          if (GTK_WIDGET_MAPPED (GTK_WIDGET (l->data)))
+            {
+              n_mapped++;
+              window = GTK_WINDOW (l->data);
+              if (window->has_focus)
+                has_focus = TRUE;
+            }
+        }
+      if (!has_focus && n_mapped > 1)
+        XSetInputFocus (xev->xany.display, GDK_WINDOW_XID (GTK_WIDGET (window)->window), RevertToPointerRoot, CurrentTime);
+
+      g_list_free (list);
       return GDK_FILTER_REMOVE;
     }
   else
