@@ -56,6 +56,8 @@
 
 #define CORE_KEYBOARD "Keyboard"
 
+#define GOK_KEYBOARD_DOCK_BORDERPIX 5
+
 /* if this flag is set then ignore the resize event (because we generated it) */
 static gint m_bIgnoreResizeEvent;
 
@@ -361,14 +363,27 @@ gok_keyboard_add_keys_from_xkb_geom (GokKeyboard *pKeyboard, XkbDescPtr kbd)
 		}
 	}
 
+	/* add a repeat next key */
+	pKey = gok_key_new (pKey, NULL, pKeyboard);
+	pKey->Style = KEYSTYLE_NORMAL;
+	pKey->Type = KEYTYPE_REPEATNEXT;
+	pKey->Target = (gchar*)g_malloc (strlen("repeat-next") + 1);
+	strcpy (pKey->Target, "repeat-next");
+	pKey->Top = row + 1;
+	pKey->Bottom = row + 2;
+	pKey->Left = 0;
+	pKey->Right = 3;
+	gok_key_add_label (pKey, _("Repeat Next Key"),NULL);
+
 	/* add a branch key for special text edit functions */
 	pKey = gok_key_new (pKey, NULL, pKeyboard);
 	pKey->Style = KEYSTYLE_GENERALDYNAMIC;
 	pKey->Type = KEYTYPE_BRANCHCOMPOSE;
-	pKey->Target = "text-operations";
+	pKey->Target = (gchar*)g_malloc (strlen("text-operations") + 1);
+	strcpy (pKey->Target, "text-operations");
 	pKey->Top = row + 1;
 	pKey->Bottom = row + 2;
-	pKey->Left = 0;
+	pKey->Left = 3;
 	pKey->Right = rightmost;
 	gok_key_add_label (pKey, _("Text Manipulation"),NULL);
 }
@@ -663,7 +678,6 @@ void gok_keyboard_delete (GokKeyboard* pKeyboard, gboolean bForce)
 		gok_spy_accessible_unref(pKeyboard->pAccessible);
 	}
 	
-	
 	/* delete all the keys on the keyboard */
 	pKey = pKeyboard->pKeyFirst;
 	while (pKey != NULL)
@@ -1247,6 +1261,8 @@ gboolean gok_keyboard_display (GokKeyboard* pKeyboard, GokKeyboard* pKeyboardCur
 	gint heightKeyHold;
 	gint widthKeyTemp;
 	gint heightKeyTemp;
+	gint borderWidth = 0;
+	gint borderHeight = 0;
 	GdkRectangle rectFrame;
 	GdkRectangle rectTemp;
 	GokButton* pGokButton;
@@ -1279,6 +1295,11 @@ gboolean gok_keyboard_display (GokKeyboard* pKeyboard, GokKeyboard* pKeyboardCur
 	/* get the "fixed container" that holds the buttons */
 	pFixedContainer = GTK_BIN(pWindowMain)->child;
 	
+	if ((gok_data_get_dock_type () != GOK_DOCK_NONE) && gok_data_get_expand ()) {
+		borderWidth = GOK_KEYBOARD_DOCK_BORDERPIX;
+		borderHeight = GOK_KEYBOARD_DOCK_BORDERPIX;
+	}
+
 	/* create all the buttons and add them to the container */
 	pKey = pKeyboard->pKeyFirst;
 	while (pKey != NULL)
@@ -1381,9 +1402,9 @@ gboolean gok_keyboard_display (GokKeyboard* pKeyboard, GokKeyboard* pKeyboardCur
 
 	/* calculate the size of the window needed for the keys */
 	widthWindow = (gok_keyboard_get_number_columns (pKeyboard) * gok_data_get_key_width()) + 
-									((gok_keyboard_get_number_columns (pKeyboard) - 1) * gok_data_get_key_spacing());
+									((gok_keyboard_get_number_columns (pKeyboard) - 1) * gok_data_get_key_spacing() + borderWidth * 2);
 	heightWindow = (gok_keyboard_get_number_rows (pKeyboard) * gok_data_get_key_height()) +
-									((gok_keyboard_get_number_rows (pKeyboard) - 1) * gok_data_get_key_spacing());
+									((gok_keyboard_get_number_rows (pKeyboard) - 1) * gok_data_get_key_spacing() + borderHeight * 2);
 
 	/* store the current key width and height */
 	widthKeyHold = gok_data_get_key_width();
@@ -1402,8 +1423,8 @@ gboolean gok_keyboard_display (GokKeyboard* pKeyboard, GokKeyboard* pKeyboardCur
 	else
 	{
 		/* TODO: how can I get the frame size before the window is shown? */
-		frameX = 5;/* this is usually true */
-		frameY = 22;/* this is usually true */
+		frameX = 0;
+		frameY = 0;
 	}
 	
 	if (gok_main_get_use_geometry() == TRUE)
@@ -1426,30 +1447,30 @@ gboolean gok_keyboard_display (GokKeyboard* pKeyboard, GokKeyboard* pKeyboardCur
 	expand = ((pKeyboard->expand == GOK_EXPAND_ALWAYS) || 
 		  (gok_data_get_expand () && (pKeyboard->expand != GOK_EXPAND_NEVER)));
 
-	if (((widthWindow + frameX) > widthMax) || 
+	if (((widthWindow + frameX + borderWidth) > widthMax) || 
 	    (pKeyboard->LayoutType == KEYBOARD_LAYOUT_FITWINDOW) || expand)
 
 	{
 		/* change the key width (for this keyboard) to fit within the screen */
-		widthKeyTemp = gok_keyboard_get_keywidth_for_window (widthMax - (frameX * 2), pKeyboard);
+		widthKeyTemp = gok_keyboard_get_keywidth_for_window (widthMax - (frameX * 2) - borderWidth, pKeyboard);
 		gok_data_set_key_width (widthKeyTemp);
 		
 		/* calculate a new window size */
 		widthWindow = (gok_keyboard_get_number_columns (pKeyboard) * gok_data_get_key_width()) + 
-									((gok_keyboard_get_number_columns (pKeyboard) - 1) * gok_data_get_key_spacing());
+									((gok_keyboard_get_number_columns (pKeyboard) - 1) * gok_data_get_key_spacing() + borderWidth * 2);
 		bKeySizeChanged = TRUE;
 	}
 	
-	if (((heightWindow + frameY) > heightMax) ||
+	if (((heightWindow + frameY - borderHeight * 2) > heightMax) ||
 	    (pKeyboard->LayoutType == KEYBOARD_LAYOUT_FITWINDOW))
 	{
 		/* change the key height (for this keyboard) to fit within the screen */
-		heightKeyTemp = gok_keyboard_get_keyheight_for_window (heightMax - frameY - frameX, pKeyboard);
+		heightKeyTemp = gok_keyboard_get_keyheight_for_window (heightMax - frameY - (borderHeight * 2), pKeyboard);
 		gok_data_set_key_height (heightKeyTemp);
 		
 		/* calculate a new window size */
 		heightWindow = (gok_keyboard_get_number_rows (pKeyboard) * gok_data_get_key_height()) +
-									((gok_keyboard_get_number_rows (pKeyboard) - 1) * gok_data_get_key_spacing());
+									((gok_keyboard_get_number_rows (pKeyboard) - 1) * gok_data_get_key_spacing() + frameY + (borderHeight * 2));
 		bKeySizeChanged = TRUE;
 	}
 	
@@ -1490,6 +1511,7 @@ void gok_keyboard_position_keys (GokKeyboard* pKeyboard, GtkWidget* pWindow)
 	gint widthKey;
 	gint heightKey;
 	gint spacingKey;
+	gint left_pad = 0, top_pad = 0;
 
 	g_assert (pKeyboard != NULL);
 	g_assert (pWindow != NULL);
@@ -1506,6 +1528,10 @@ void gok_keyboard_position_keys (GokKeyboard* pKeyboard, GtkWidget* pWindow)
 
 	/* loop through all the keys */
 	pKey = pKeyboard->pKeyFirst;
+	if ((gok_data_get_dock_type () != GOK_DOCK_NONE) && gok_data_get_expand ()) {
+		left_pad = GOK_KEYBOARD_DOCK_BORDERPIX;
+		top_pad = GOK_KEYBOARD_DOCK_BORDERPIX;
+	}
 	while (pKey != NULL)
 	{
 		/* change the size of the button */
@@ -1516,8 +1542,8 @@ void gok_keyboard_position_keys (GokKeyboard* pKeyboard, GtkWidget* pWindow)
 		gtk_widget_set_size_request (pKey->pButton, width, height);
 
 		/* position the button */
-		left = pKey->Left * (widthKey + spacingKey);
-		top = pKey->Top * (heightKey + spacingKey);
+		left = pKey->Left * (widthKey + spacingKey) + left_pad;
+		top = pKey->Top * (heightKey + spacingKey) + top_pad;
 		
 		/* if the button has been previously 'put' then 'move' it */
 		if (gtk_widget_get_parent (pKey->pButton) == NULL)
@@ -2482,6 +2508,7 @@ gok_keyboard_about (GokKey *pKey)
 		"David Bolter <david.bolter@utoronto.ca>",
 		"Bill Haneman <bill.haneman@sun.com>",
 		"Chris Ridpath <chris.ridpath@utoronto.ca>",
+		"Gnome Accessibility <gnome-accessibility-devel@gnome.org>",
 		NULL
 	};
 
@@ -2500,7 +2527,7 @@ gok_keyboard_about (GokKey *pKey)
 	}
         
 	file = gnome_program_locate_file (NULL, GNOME_FILE_DOMAIN_APP_DATADIR, 
-					  "gok/goklogo.png", 
+					  "gok.png", 
 					  FALSE, NULL);
 	if (file) {
 		pixbuf = gdk_pixbuf_new_from_file (file, &error);
@@ -2683,7 +2710,7 @@ GokKey* gok_keyboard_output_selectedkey (void)
 	/* is a key selected? */
 	if (pKeySelected == NULL)
 	{
-		gok_log_x ("Currently selected key is NULL!");
+		gok_log ("Currently selected key is NULL!");
 		gok_log_leave();
 		return pKeySelected;
 	}
@@ -2760,7 +2787,6 @@ GokKey* gok_keyboard_output_key(GokKey* pKey)
 			break;
 
 		case KEYTYPE_BRANCHBACK:
-gok_log_x("BRANCH BACK %s", pKey->pLabel->Text);			
 		case KEYTYPE_BRANCHMENUS:
 		case KEYTYPE_BRANCHTOOLBARS:
 		case KEYTYPE_BRANCHGUI:
