@@ -40,15 +40,14 @@
 #include "gok-modifier.h"
 #include "gok-data.h"
 #include "gok-modifier-keymasks.h"
+#include "gok-gconf-keys.h"
 
 /*
- * these definitions allow us to tweak the efficiency/accuracy of the
- * gui search
+ * NOTE: we can tweak the efficiency/accuracy of the
+ * gui search by adjusting using the spy/gui_search_depth and 
+ * spy/gui_search_breadth gconf keys.
  */
 
-#define GOKSPY_GUI_BREADTH 999
-#define GOKSPY_GUI_DEPTH 999
-
 /*
  * NOTES
  *
@@ -97,7 +96,7 @@ static AccessibleNode*  gok_spy_append_node (AccessibleNode* pnode,
 static void gok_spy_modifier_listener(const AccessibleEvent *event,
                                       void *user_data);
 
-static gboolean gok_spy_worth_searching (Accessible* accessible);
+static gboolean gok_spy_worth_searching (Accessible* accessible, AccessibleRole role);
 
 static void gok_spy_window_activate_listener (const AccessibleEvent *event,
                                               void *user_data);
@@ -118,7 +117,7 @@ static void* m_ptheMouseButtonListener;
 static Accessible* m_ptheWindowAccessible;
 
 /* pointer to the current accessible with editable text */
-static Accessible* m_ptheAccessibleWithEditableText;
+static Accessible* m_ptheAccessibleWithText;
 
 static AccessibleEventListener* focusListener;
 static AccessibleEventListener* defunctnessListener;
@@ -131,6 +130,9 @@ static gint explicitrefs;
 static gint explicitunrefs;
 static gint implicitrefs;
 static gulong keyboardmods;
+static gboolean spyshutdown = FALSE;/* this global could be removed if we have a
+						way of removing our idle handler in gok_spy_stop */
+
 
 /* 
  * for debugging
@@ -145,6 +147,7 @@ void gok_spy_accessible_ref(Accessible* accessible)
 	{
 		explicitrefs++;
 		Accessible_ref (accessible);
+#if defined(ENABLE_LOGGING_NORMAL)
 		s = Accessible_getName (accessible);
 		if (s == NULL)
 		{
@@ -155,6 +158,7 @@ void gok_spy_accessible_ref(Accessible* accessible)
 			gok_log ("%#x %s", accessible, s);
 			SPI_freeString (s);
 		}
+#endif
 	}
 	else
 	{
@@ -172,6 +176,7 @@ void gok_spy_accessible_implicit_ref(Accessible* accessible)
 	if (accessible != NULL)
 	{
 		implicitrefs++;
+#if defined(ENABLE_LOGGING_NORMAL)
 		s = Accessible_getName (accessible);
 		if (s == NULL)
 		{
@@ -182,6 +187,7 @@ void gok_spy_accessible_implicit_ref(Accessible* accessible)
 			gok_log ("%#x %s", accessible, s);
 			SPI_freeString (s);
 		}
+#endif
 	}
 	else
 	{
@@ -198,6 +204,7 @@ void gok_spy_accessible_unref(Accessible* accessible)
 
 	if (accessible != NULL)
 	{
+#if defined(ENABLE_LOGGING_NORMAL)
 		s = Accessible_getName (accessible);
 		if (s == NULL)
 		{
@@ -208,6 +215,7 @@ void gok_spy_accessible_unref(Accessible* accessible)
 			gok_log ("%#x %s", accessible, s);
 			SPI_freeString (s);
 		}
+#endif
 		explicitunrefs++;
 		Accessible_unref (accessible);
 	}
@@ -250,6 +258,13 @@ gboolean gok_spy_accessible_is_okay(Accessible* accessible)
 	return returncode;
 }
 
+static gboolean
+gok_spy_idle_handler (gpointer data)
+{
+	return gok_spy_check_queues ();
+}
+
+
 /**
  * gok_spy_open:
  *
@@ -272,7 +287,7 @@ void gok_spy_open()
 		m_ptheAppChangeListener = NULL;
 		m_ptheWindowChangeListener = NULL;
 		m_ptheWindowAccessible = NULL;
-		m_ptheAccessibleWithEditableText = NULL;
+		m_ptheAccessibleWithText = NULL;
 
 		focusListener = SPI_createAccessibleEventListener (
 			gok_spy_focus_listener, NULL);
@@ -315,6 +330,62 @@ void gok_spy_open()
 	gok_log_leave ();
 }
 
+/** 
+ * gok_spy_stop:
+ *
+ * Call this to unhook all the gok_spy internal listeners. Do this as as the
+ * first thing when shutting down the application.
+ */
+void gok_spy_stop (void)
+{
+	gboolean result = FALSE;	
+	
+	spyshutdown = TRUE;
+	
+	if (focusListener != NULL) 
+	{
+		result = SPI_deregisterGlobalEventListenerAll(focusListener);
+		gok_log("deregistering focus listener returned: %d",result);
+		AccessibleEventListener_unref(focusListener);
+		focusListener = NULL;
+	}
+	if (defunctnessListener != NULL) 
+	{
+		result = SPI_deregisterGlobalEventListenerAll(defunctnessListener);
+		gok_log("deregistering defunctness listener returned: %d",result);
+		AccessibleEventListener_unref(defunctnessListener);
+		defunctnessListener = NULL;
+	}
+	if (mouseListener != NULL)
+	{
+		result = SPI_deregisterDeviceEventListener (mouseListener, NULL);
+		gok_log("deregistering mouse listener returned: %d",result);
+		AccessibleDeviceListener_unref(mouseListener);
+		mouseListener = NULL;
+	}
+	if (modifierListener != NULL)
+	{
+		result = SPI_deregisterGlobalEventListenerAll (modifierListener);
+		gok_log("deregistering modifier listener returned: %d",	result);
+		AccessibleEventListener_unref(modifierListener);
+		modifierListener = NULL;
+	}
+	if (windowActivateListener != NULL)
+	{
+		result = SPI_deregisterGlobalEventListenerAll (windowActivateListener);
+		gok_log("deregistering window activate listener returned: %d",result);
+		AccessibleDeviceListener_unref(windowActivateListener);
+		windowActivateListener = NULL;
+	}
+	if (windowDeactivateListener != NULL)
+	{
+		result= SPI_deregisterGlobalEventListenerAll (windowDeactivateListener);
+		gok_log("deregistering window deactivate listener returned: %d",result);
+		AccessibleEventListener_unref(windowDeactivateListener );
+		windowDeactivateListener = NULL;
+	}
+}
+	
 /** 
  * gok_spy_close:
  *
@@ -323,7 +394,6 @@ void gok_spy_open()
 void gok_spy_close(void)
 {
 	EventNode* en;
-	gboolean result = FALSE;	
 	
 	gok_log_enter ();
 
@@ -331,37 +401,11 @@ void gok_spy_close(void)
 	{
 		m_gokSpyOpen = FALSE;
 		
-		if (focusListener != NULL) 
-		{
-			result = SPI_deregisterGlobalEventListenerAll(focusListener);
-			gok_log("deregistering focus listener returned: %d",result);
-		}
-		if (defunctnessListener != NULL) 
-		{
-			result = SPI_deregisterGlobalEventListenerAll(defunctnessListener);
-			gok_log("deregistering defunctness listener returned: %d",result);
-		}
-		if (mouseListener != NULL)
-		{
-			result = SPI_deregisterDeviceEventListener (mouseListener, NULL);
-			gok_log("deregistering mouse listener returned: %d",result);
-		}
-		if (modifierListener != NULL)
-		{
-			result = SPI_deregisterGlobalEventListenerAll(
-				modifierListener);
-			gok_log("deregistering modifier listener returned: %d",
-				result);
-		}
-	
-		AccessibleEventListener_unref(focusListener);
-		AccessibleEventListener_unref(defunctnessListener);
-		AccessibleDeviceListener_unref(mouseListener);
-		AccessibleEventListener_unref(modifierListener);
-
-		if (m_ptheWindowAccessible != NULL) gok_spy_accessible_unref(m_ptheWindowAccessible);
-		if (m_ptheAccessibleWithEditableText != NULL) gok_spy_accessible_unref(m_ptheAccessibleWithEditableText);
+		gok_spy_stop ();
 		
+		if (m_ptheWindowAccessible != NULL) gok_spy_accessible_unref(m_ptheWindowAccessible);
+		if (m_ptheAccessibleWithText != NULL) gok_spy_accessible_unref(m_ptheAccessibleWithText);
+
 		gok_log("accessible reference balance is  [%d] (should be zero)",explicitrefs + implicitrefs - explicitunrefs);
 		gok_log("explicit accessible references   [%d]",explicitrefs);
 		gok_log("explicit accessible dereferences [%d]",explicitunrefs);
@@ -512,13 +556,13 @@ void gok_spy_free (AccessibleNode* pNode)
 }
 
 /** 
-* gok_spy_get_accessibleWithEditableText
+* gok_spy_get_accessibleWithText
 *
 * accessor
 **/
-Accessible* gok_spy_get_accessibleWithEditableText()
+Accessible* gok_spy_get_accessibleWithText()
 {
-	return m_ptheAccessibleWithEditableText;
+	return m_ptheAccessibleWithText;
 }
 
 /** 
@@ -537,8 +581,13 @@ gboolean gok_spy_has_child (Accessible* accessible, GokSpySearchType type, Acces
 	gok_log_enter();
 	if (gok_spy_accessible_is_okay(accessible) == TRUE)
 	{
-		gok_spy_has_child_aux (accessible, type, role, 
-				       GOKSPY_GUI_BREADTH, GOKSPY_GUI_DEPTH, 0, &bfound);
+		gint breadth;
+		gint depth;
+		GConfClient* client = gok_data_get_gconf_client ();
+		gok_gconf_get_int ( client, GOK_GCONF_SPY_SEARCH_BREADTH, &breadth );
+		gok_gconf_get_int ( client, GOK_GCONF_SPY_SEARCH_DEPTH, &depth );
+		gok_spy_has_child_aux (accessible, type, role, breadth, depth, 
+			0, &bfound);
 	}
 	else
 	{
@@ -560,12 +609,18 @@ gboolean gok_spy_check_queues(void)
 	char *s;
 
 	gok_log_enter();
+	
+	if (spyshutdown) {
+		return FALSE;
+	}
+	
 	while (eventQueue && g_queue_is_empty(eventQueue) == FALSE)
 	{
 		en = (EventNode*)g_queue_pop_head(eventQueue);
 
 		gok_log ("eventQueue->length = %d", eventQueue->length);
 
+#if defined(ENABLE_LOGGING_NORMAL)
 		if (en->paccessible != NULL)
 		{
 			s = Accessible_getName (en->paccessible);
@@ -579,6 +634,7 @@ gboolean gok_spy_check_queues(void)
 				SPI_freeString (s);
 			}
 		}
+#endif
 
 		switch (en->type)
 		{
@@ -644,11 +700,13 @@ gboolean gok_spy_check_queues(void)
 */ 
 gboolean gok_spy_check_window(AccessibleRole role)
 {
+	/* TODO - improve efficiency here? */
 	if ((role == SPI_ROLE_WINDOW) ||
 	    (role == SPI_ROLE_DIALOG) ||
 	    (role == SPI_ROLE_FILE_CHOOSER) ||
 	    (role == SPI_ROLE_FRAME) ||	
-	    (role == SPI_ROLE_DESKTOP_FRAME))
+	    (role == SPI_ROLE_DESKTOP_FRAME) ||
+		(role == SPI_ROLE_FONT_CHOOSER))
 	{
 		return TRUE;
 	}
@@ -656,20 +714,51 @@ gboolean gok_spy_check_window(AccessibleRole role)
 }
 
 /** 
-* gok_spy_worth_searching:
-*
-* @accessible: the accessible (possibly a parent) to examine for worthiness.
-*
-* This function decides if the accessible might have a subtree worth searching.
-*
-* Returns: boolean success. 
+ * gok_spy_worth_searching:
+ *
+ * @accessible: the accessible (possibly a parent) to examine for worthiness.
+ * @role: the role we are searching for.
+ *
+ * This function decides if the accessible might have a subtree worth searching.
+ *
+ * Returns: boolean success. 
 */ 
-gboolean gok_spy_worth_searching (Accessible* accessible)
+gboolean gok_spy_worth_searching (Accessible* accessible, AccessibleRole role)
 {
 	gboolean worthiness;
 	AccessibleStateSet* ass = NULL;
-	worthiness = TRUE;
+
 	gok_log_enter();
+	
+	if (gok_spy_is_menu_role (role))
+	{		
+		switch (Accessible_getRole(accessible))
+		{
+			/* using this breaks mozilla menus:
+			case SPI_ROLE_TOOL_BAR: */
+			case SPI_ROLE_STATUS_BAR:
+			case SPI_ROLE_COMBO_BOX:
+				/* add cases here */
+				gok_log("aborting menu search");
+				gok_log_leave();
+				return FALSE;
+		}
+	}
+	else
+	{
+		switch (Accessible_getRole(accessible))
+		{
+			case SPI_ROLE_MENU_BAR:
+			case SPI_ROLE_MENU:
+			case SPI_ROLE_COMBO_BOX:
+				/* add cases here */
+				gok_log("aborting search");
+				gok_log_leave();
+				return FALSE;
+		}
+	}
+	
+	worthiness = TRUE;
 	ass = Accessible_getStateSet(accessible);
 	if (ass != NULL)
 	{
@@ -717,7 +806,7 @@ gboolean gok_spy_has_child_aux (Accessible* accessible,
     if (accessible == NULL) return *found; 
 	if (cdepth++ == depth) return *found;
 	
-	if (gok_spy_worth_searching(accessible) == FALSE) return *found;
+	if (gok_spy_worth_searching(accessible, role) == FALSE) return *found;
 	
     child_count = Accessible_getChildCount (accessible);
     
@@ -809,10 +898,10 @@ gok_spy_is_ui (Accessible *accessible, AccessibleRole role)
 	case SPI_ROLE_CHECK_BOX:
 	case SPI_ROLE_COMBO_BOX:
 	case SPI_ROLE_RADIO_BUTTON:
+	case SPI_ROLE_PAGE_TAB:
 	case SPI_ROLE_TOGGLE_BUTTON:
 		if (gok_spy_is_visible (accessible))
 			is_interesting_ui_component = TRUE;
-		
 	default:
 		break;
 	}
@@ -853,7 +942,7 @@ void gok_spy_find_children (AccessibleNode* pNode, GokSpySearchType type, Access
 
 	pAccessibleParent = pNode->paccessible; 
 
-	if (gok_spy_worth_searching(pAccessibleParent) == FALSE) return;
+	if (gok_spy_worth_searching(pAccessibleParent, role) == FALSE) return;
 
 	child_count = Accessible_getChildCount (pAccessibleParent);
 
@@ -892,6 +981,10 @@ void gok_spy_find_children (AccessibleNode* pNode, GokSpySearchType type, Access
 			     (Accessible_isEditableText(pAccessibleChild))))
 			{
 				gok_spy_append_node (pNode, pAccessibleChild);
+				if (childrole == SPI_ROLE_PAGE_TAB) {
+					pNode->paccessible = pAccessibleChild;
+					gok_spy_find_children (pNode, type, role);
+				}
 			}
 			else /* search through the children of this child */
 			{
@@ -1021,7 +1114,8 @@ AccessibleNode* gok_spy_append_node (AccessibleNode* pNode,
 			   * on the link!  But at the moment the link objects are not
 			   * actionable, so we have to fake it.
 			   */
-				gok_spy_add_node (pNode, pAccessible, i, linkname);
+				gok_spy_add_node (pNode, pAccessible, i, g_strdup (linkname));
+				SPI_freeString (linkname);
 			}
 			Accessible_unref (link_object);
 		}
@@ -1065,6 +1159,7 @@ AccessibleNode* gok_spy_append_node (AccessibleNode* pNode,
 					}
 					break; /* from while loop */
 				}
+				i++;
 			}
 		}
 		
@@ -1174,7 +1269,7 @@ void gok_spy_defunctness_listener (const AccessibleEvent *event, void *user_data
 		en->type = GOKSPY_DEFUNCT_EVENT;
 		g_queue_push_tail ( eventQueue, en );
 		gok_spy_add_idle_handler ();
-		gok_log_x("our window is gone!");
+		gok_log ("Our target window is now defunct!");
 	}
 	
 	gok_log_leave();
@@ -1237,28 +1332,28 @@ void gok_spy_process_accessible (Accessible* accessible)
 	gok_spy_accessible_ref(accessible); 
 
 	/*
-	 *  check to see if focus is on an editable text object 
+	 *  check to see if focus is on a text object 
 	 */
-	if (Accessible_isEditableText(accessible) == TRUE)
+	if (Accessible_isText(accessible) == TRUE)
 	{
-		gok_log("this thing has an editable text interface");
-		if (accessible != m_ptheAccessibleWithEditableText)
+		gok_log("this thing has a text interface");
+		if (accessible != m_ptheAccessibleWithText)
 		{
 			gok_spy_accessible_ref(accessible); 
-			gok_spy_accessible_unref(m_ptheAccessibleWithEditableText);
-			m_ptheAccessibleWithEditableText = accessible;
-			gok_log("found a (new) editable text interface"); 
+			gok_spy_accessible_unref(m_ptheAccessibleWithText);
+			m_ptheAccessibleWithText = accessible;
+			gok_log("found a (new) text interface"); 
 		}
 	}
 	else
 	{
-		gok_log("no editable text interface on this thing");
+		gok_log("no text interface on this thing");
 		/* we might not want to do this - needs user testing */
-		if (m_ptheAccessibleWithEditableText != NULL)
+		if (m_ptheAccessibleWithText != NULL)
 		{
-			gok_spy_accessible_unref(m_ptheAccessibleWithEditableText);
+			gok_spy_accessible_unref(m_ptheAccessibleWithText);
 		}
-		m_ptheAccessibleWithEditableText = NULL;
+		m_ptheAccessibleWithText = NULL;
 	}
 
 	/* 
@@ -1419,12 +1514,6 @@ void gok_spy_modifier_listener(const AccessibleEvent *event, void *user_data)
 	gok_log_leave();
 }
 
-static gboolean
-gok_spy_idle_handler (gpointer data)
-{
-	return gok_spy_check_queues ();
-}
-
 /** 
  * gok_spy_add_idle_handler:
  *
@@ -1435,6 +1524,7 @@ void gok_spy_add_idle_handler (void)
 	g_idle_add (gok_spy_idle_handler, NULL);
 }
 
+
 /*
 gboolean gok_spy_is_desktop(Accessible* pAccessible)
 {
