@@ -47,6 +47,7 @@ typedef enum {
 
 void gok_spy_open(void);        /* call this first (user must call SPI_init first) */
 void gok_spy_close(void);       /* call this when shutting down */ 
+void gok_spy_stop (void);       /* call this first, as soon as you start shutting down */
  
 // listener support 
 void gok_spy_register_appchangelistener(AppChangeListener* callback); 
@@ -68,7 +69,7 @@ AccessibleNode* gok_spy_get_ui_list( Accessible* paccessible);
 AccessibleNode* gok_spy_refresh( AccessibleNode* plist); 
 void gok_spy_free( AccessibleNode* plist ); /* call this to free up an uneeded list's memory */ 
 
-Accessible* gok_spy_get_accessibleWithEditableText(void);
+Accessible* gok_spy_get_accessibleWithText(void);
 
 gboolean gok_spy_has_child (Accessible* accessible, GokSpySearchType type, AccessibleRole role); 
 
