@@ -75,10 +75,12 @@
 #define GOK_ERROR_DISPLAY_SCAN_MAIN   5
 #define GOK_ERROR_ACTION_OPEN         6
 #define GOK_ERROR_FEEDBACK_OPEN       7
+#define GOK_ERROR_NO_XKB_EXTENSION    8
 
 #define GOKMAINKEYBOARDNAME "main"
 
 #define KEYBOARD_ACCESSIBILITY_STICKY_KEYS_KEY "/desktop/gnome/accessibility/keyboard/stickykeys_enable"
+#define GCONF_ACCESSIBILITY_KEY "/desktop/gnome/interface/accessibility"
 
 /* pointer to the first keyboard in the list of keyboards */
 static GokKeyboard* m_pKeyboardFirst = NULL;
@@ -218,11 +220,13 @@ static struct poptOption options[] = {
 	{NULL, '\0', 0, NULL, 0, NULL, NULL}
 };
 
+static int segfaults = 0;
 /* private */
 static int gok_sig_handler(int sig)
 {
 	switch (sig) {
 	case SIGSEGV:
+		segfaults++;
 		fprintf (stderr, "gok: Segmentation fault\n");
 		break;
 	case SIGTERM:
@@ -232,8 +236,9 @@ static int gok_sig_handler(int sig)
 	default:
 		break;
 	}
-
-	gok_main_close();
+	
+	if (segfaults == 1)
+		gok_main_close();
 
 	_exit(1);
 	return -1;  /* this line gets rid of compiler warning */ 
@@ -367,6 +372,13 @@ gint gok_main_open(gint argc, gchar *argv[])
 		SPI_exit();
 		return 1;
 	}
+	
+	/* check for xkb extension */
+	if (!gok_main_has_xkb_extension())
+	{
+		gok_main_display_error (_("XKB extension is required."));
+		return GOK_ERROR_NO_XKB_EXTENSION;
+	}		
 
 	/* read the keyboards */
 	gok_main_read_keyboards ();
@@ -435,7 +447,10 @@ gint gok_main_open(gint argc, gchar *argv[])
 		mainkeyboardname = (gchar*) g_malloc (strlen (GOKMAINKEYBOARDNAME) + 1);
 		strcpy (mainkeyboardname, (gchar *) GOKMAINKEYBOARDNAME);
 	}
-		
+	
+
+	gok_main_check_accessibility();
+	
 	/* display the "main" keyboard */
 	if (gok_main_display_scan_main () == FALSE)
 	{
@@ -562,6 +577,7 @@ Accessible* gok_main_get_foreground_window_accessible()
 	return m_pForegroundWindowAccessible;
 }
 
+
 /**
 * gok_main_window_change_listener
 * @pAccessible: Pointer to the foreground window's accessible interface.
@@ -1765,6 +1781,8 @@ void gok_main_close()
 	gok_spy_deregister_windowchangelistener ((void *)gok_main_window_change_listener);
 	/* the call to deregister_mousebuttonlistener is in on_window1_destroy */
 	
+	gok_spy_stop();
+	
 	gok_log("BEFORE DELETING KEYBOARDS");
 	gok_log("Keyboards news - deletes: [%d]",gok_keyboard_get_keyboards());
 	gok_log("Stack pushes - pops:      [%d]",gok_branchbackstack_pushes_minus_pops());
@@ -1799,6 +1817,8 @@ void gok_main_close()
 	gok_control_button_callback_close();
 	gok_wordcomplete_close();
 	gok_predictor_close(m_pCommandPredictor);
+	
+	_exit(0);
 }
 
 /**
@@ -1947,6 +1967,25 @@ gok_main_get_login (void)
 	return bLoginOption;
 }
 
+
+/**
+* gok_main_has_xkb_extension
+*
+* Call this to see if xkb is enabled.
+* 
+* returns: gboolean
+*/
+gboolean
+gok_main_has_xkb_extension ()
+{
+	/* TODO/revisit - is this a good check? */
+	if (gok_keyboard_get_xkb_desc() != NULL)
+	{
+		return TRUE;
+	}
+	return FALSE;
+}
+
 gboolean
 gok_main_check_sticky_keys (GtkWidget *widget)
 {
@@ -1982,7 +2021,7 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 				xkb_ok = TRUE;
 				dialog = gtk_message_dialog_new (GTK_WINDOW (widget),
 								 GTK_DIALOG_DESTROY_WITH_PARENT,
-								 GTK_MESSAGE_WARNING,
+								 GTK_MESSAGE_INFO,
 								 GTK_BUTTONS_OK,
 								 "GOK has enabled Sticky Keys, which it requires.\n");
 				
@@ -2013,3 +2052,90 @@ gok_main_check_sticky_keys (GtkWidget *widget)
 	return TRUE;
 }
 
+
+
+static GtkWidget* acd = NULL; /* accessibility check dialog */
+
+void check_accessibility_cb ( GObject* o, gpointer* data )
+{
+	gtk_widget_destroy(acd);	
+	
+	if (strcmp((gchar*)data,"logout") == 0) {
+		gok_gconf_set_bool ( gok_data_get_gconf_client(),
+			GCONF_ACCESSIBILITY_KEY, TRUE );
+			gok_main_close();  /* FIXME: need to logout */
+			_exit (0);
+	}
+	else if (strcmp((gchar*)data,"quit") == 0) {
+		gok_main_close();
+		_exit (0);
+	}
+	else if (strcmp((gchar*)data,"continue") == 0) {
+		/* maybe change gok somehow to show user weakened status */
+	}
+
+}
+
+void
+gok_main_check_accessibility ()
+{
+    gboolean     accessibility_on;
+    GConfClient* client;
+	
+	client = gok_data_get_gconf_client();
+
+    /* check if accessibility flag is TRUE */
+	gok_gconf_get_bool ( client, GCONF_ACCESSIBILITY_KEY, &accessibility_on );
+        
+    if (!accessibility_on) {
+		GtkWidget*	button;
+		gint         response_id;
+		gboolean 	returnCode = TRUE;
+		
+		acd = gtk_message_dialog_new (
+			GTK_WINDOW(m_pWindowMain), 
+			GTK_DIALOG_DESTROY_WITH_PARENT,
+			GTK_MESSAGE_WARNING,
+			GTK_BUTTONS_NONE,			
+			_("Assistive Technology Support Is Not Enabled." 
+		"\n\n" 
+		"You can start GOK without enabling support for assistive technologies. "
+		"However, some of the features of the application might not be available." 
+		"\n\n"
+		"To enable support for assistive technologies "
+		"and log in to a new session with the change enabled, "
+		"click "
+		"Enable and Log Out" "."
+		"\n\n"
+		"To continue using GOK, " 
+		"click "
+		"Continue" "."
+		"\n\n"
+		"To quit GOK, " 
+		"click " 
+		"Close" "." 
+		"\n\n"));
+		   
+		button = gtk_button_new_with_label(_("Enable and Log Out"));
+		gtk_container_add (GTK_CONTAINER (GTK_DIALOG(acd)->action_area),
+						  button);
+		g_signal_connect(G_OBJECT (button), "clicked", 
+			G_CALLBACK (check_accessibility_cb), (gpointer) "logout");
+			
+		button = gtk_button_new_from_stock(GTK_STOCK_CLOSE);
+		gtk_container_add (GTK_CONTAINER (GTK_DIALOG(acd)->action_area),
+						  button);
+		g_signal_connect(G_OBJECT (button), "clicked", 
+			G_CALLBACK (check_accessibility_cb), (gpointer) "quit");
+
+		button = gtk_button_new_with_label(_("Continue"));
+		gtk_container_add (GTK_CONTAINER (GTK_DIALOG(acd)->action_area),
+						  button);
+		g_signal_connect(G_OBJECT (button), "clicked", 
+			G_CALLBACK (check_accessibility_cb), (gpointer) "continue");
+						  
+
+		gtk_widget_show_all (acd);
+		
+	}
+}
